@@ -133,7 +133,8 @@ class WDM():
         self.openfiles = {}
 
     def wmsgop(self):
-        # WMSGOP is a simple open of the message file
+        ''' WMSGOP is a simple open of the message file.
+        '''
         afilename = os.path.join(sys.prefix,
                                  'share',
                                  'wdmtoolbox',
@@ -141,17 +142,20 @@ class WDM():
         return self._open(afilename, 100, ronwfg=1)
 
     def dateconverter(self, datestr):
+        ''' Extracts all of the grouped numbers out of a string
+        to create an array suitable for dates and times.
+        '''
         words = re.findall(r'\d+', str(datestr))
         words = [int(i) for i in words]
         dtime = [1900, 1, 1, 0, 0, 0]
         dtime[:len(words)] = words
-        return dtime
+        return pd.np.array(dtime)
 
     def _open(self, wdname, wdmsfl, ronwfg=0):
         ''' Private method to open WDM file.
         '''
+        wdname = wdname.strip()
         if wdname not in self.openfiles:
-            wdname = wdname.strip()
             if ronwfg == 1:
                 if not os.path.exists(wdname):
                     raise ValueError('''
@@ -169,9 +173,14 @@ class WDM():
         return wdmsfl
 
     def _retcode_check(self, retcode, additional_info=' '):
+        ''' Central place to run through the return code.
+        '''
+        if retcode == 0:
+            return
         retcode_dict = {
             -1: 'non specific error on WDM file open',
-            -4: 'copy/update failed due to data overlap problem - part of source needed',
+            -4: '''copy/update failed due to data overlap problem - part of
+    source needed''',
             -5: 'copy/update failed due to data overlap problem',
             -6: 'no data present',
             -8: 'bad dates',
@@ -179,8 +188,10 @@ class WDM():
             -10: 'no date in this group',
             -11: 'no non-missing data, data has not started yet',
             -14: 'data specified not within valid range for data set',
-            -15: 'time units and time step must match label exactly with VBTIME = 1',
-            -20: 'problem with on or more of GPGLG, DXX, NVAL, QUALVL, LTSTEP, LTUNIT',
+            -15: '''time units and time step must match label exactly with
+    VBTIME = 1''',
+            -20: '''problem with one or more of
+    GPGLG, DXX, NVAL, QUALVL, LTSTEP, LTUNIT''',
             -21: 'data from WDM does not match expected date',
             -23: 'not a valid table',
             -24: 'not a valid associated table',
@@ -204,7 +215,7 @@ class WDM():
             -45: 'types do not match',
             -46: 'bad space time group specification parameter',
             -47: 'bad direction flag',
-            -48: 'conflicting spec of space time dim and number of timeseries data sets',
+            -48: 'conflicting spec of space time dim and # of ts data sets',
             -49: 'group does not exist',
             -50: 'requested attributes missing from this data set',
             -51: 'no space for another DLG',
@@ -253,6 +264,8 @@ class WDM():
 '''.format(retcode, additional_info))
 
     def renumber_dsn(self, wdmpath, odsn, ndsn):
+        ''' Will renumber the odsn to the ndsn.
+        '''
         odsn = int(odsn)
         ndsn = int(ndsn)
 
@@ -265,6 +278,8 @@ class WDM():
         self._retcode_check(retcode, additional_info='wddsrn')
 
     def delete_dsn(self, wdmpath, dsn):
+        ''' Function to delete a DSN.
+        '''
         dsn = int(dsn)
 
         wdmfp = self._open(wdmpath, 101)
@@ -276,6 +291,8 @@ class WDM():
         self._close(wdmpath)
 
     def copydsnlabel(self, inwdmpath, indsn, outwdmpath, outdsn):
+        ''' Will copy a complete DSN label from one DSN to another.
+        '''
         assert inwdmpath != outwdmpath
         indsn = int(indsn)
         outdsn = int(outdsn)
@@ -293,6 +310,8 @@ class WDM():
         self._retcode_check(retcode, additional_info='wddscl')
 
     def describe_dsn(self, wdmpath, dsn):
+        ''' Will collect some metadata about the DSN.
+        '''
         wdmfp = self._open(wdmpath, 101, ronwfg=1)
         if self.wdckdt(wdmfp, dsn) == 0:
             raise DSNDoesNotExist(dsn)
@@ -420,7 +439,7 @@ class WDM():
         elif os.path.exists(wdmpath):
             raise WDMFileExists(wdmpath)
         ronwfg = 2
-        wdmfp = self._open(wdmpath, 101, ronwfg=ronwfg)
+        self._open(wdmpath, 101, ronwfg=ronwfg)
         self._close(wdmpath)
 
     def create_new_dsn(self, wdmpath, dsn, tstype='', base_year=1900, tcode=4,
@@ -434,7 +453,7 @@ class WDM():
             raise DSNExistsError(dsn)
 
         # Parameters for wdlbax taken from ATCTSfile/clsTSerWDM.cls
-        psa = self.wdlbax(
+        self.wdlbax(
             wdmfp,
             dsn,
             1,    # DSTYPE - always 1 for time series
@@ -573,10 +592,36 @@ class WDM():
 
         if start_date is not None:
             start_date = self.dateconverter(start_date)
-            llsdat = start_date
+            if datetime.datetime(*start_date) >= datetime.datetime(*llsdat):
+                llsdat = start_date
+            if datetime.datetime(*start_date) > datetime.datetime(*lledat):
+                raise ValueError('''
+*
+*   The requested start date ({0}) is after the end date ({1})
+*   of the time series in the WDM file.
+*
+'''.format(datetime.datetime(*start_date), datetime.datetime(*lledat)))
+        else:
+            start_date = llsdat
         if end_date is not None:
             end_date = self.dateconverter(end_date)
-            lledat = end_date
+            if datetime.datetime(*end_date) <= datetime.datetime(*lledat):
+                lledat = end_date
+            if datetime.datetime(*end_date) < datetime.datetime(*llsdat):
+                raise ValueError('''
+*
+*   The requested end date ({0}) is before the start date ({1})
+*   of the time series in the WDM file.
+*
+'''.format(datetime.datetime(*end_date), datetime.datetime(*llsdat)))
+        else:
+            end_date = lledat
+        if datetime.datetime(*end_date) < datetime.datetime(*start_date):
+            raise ValueError('''
+*
+*   The end date ({0}) is less than the start date ({1}).
+*
+'''.format(datetime.datetime(*end_date), datetime.datetime(*start_date)))
 
         # Determine the number of values (ITERM) from LLSDAT to LLEDAT
         iterm = self.timdif(
@@ -602,12 +647,7 @@ class WDM():
         self._retcode_check(retcode, additional_info='wdtget')
 
         # Find the begining in datetime.datetime format
-        tstart = datetime.datetime(llsdat[0],
-                                   llsdat[1],
-                                   llsdat[2],
-                                   llsdat[3],
-                                   llsdat[4],
-                                   llsdat[5])
+        tstart = datetime.datetime(*llsdat)
 
         # Convert time series to pandas DataFrame
         index = pd.date_range(
@@ -621,6 +661,14 @@ class WDM():
                 index=index,
                 name='{0}_DSN_{1}'.format(
                     os.path.basename(wdmpath), dsn)), dtype=pd.np.float64)
+        if (datetime.datetime(*start_date) != datetime.datetime(*llsdat) or
+            datetime.datetime(*end_date) != datetime.datetime(*lledat)):
+            rindex = pd.date_range(datetime.datetime(*start_date),
+                                   datetime.datetime(*end_date),
+                                   freq='{0:d}{1}'.format(tstep,
+                                                          MAPTCODE[tcode]))
+            tmpval = tmpval.reindex(rindex)
+
         tmpval.replace(tsfill, pd.np.nan, inplace=True)
         tmpval.index.name = 'Datetime'
         return tmpval
@@ -633,6 +681,7 @@ class WDM():
     def _close(self, wdmpath):
         ''' Close the WDM file.
         '''
+        wdmpath = wdmpath.strip()
         if wdmpath in self.openfiles:
             retcode = self.wdflcl(self.openfiles[wdmpath])
             self._retcode_check(retcode, additional_info='wdflcl')
@@ -646,6 +695,11 @@ if __name__ == '__main__':
         fname = r'c:\test.wdm'
     wdm_obj.create_new_wdm(fname, overwrite=True)
     listonumbers = [34.2, 35.0, 36.9, 38.2, 40.2, 20.1, 18.4, 23.6]
-    wdm_obj.create_new_dsn(fname, 1003, tstype='EXAM', scenario='OBSERVED', tcode=4, location='EXAMPLE')
+    wdm_obj.create_new_dsn(fname,
+                           1003,
+                           tstype='EXAM',
+                           scenario='OBSERVED',
+                           tcode=4,
+                           location='EXAMPLE')
     wdm_obj.write_dsn(fname, 1003, listonumbers, datetime.datetime(2000, 1, 1))
     print(wdm_obj.read_dsn_por(fname, 1003))
